//#include <iostream.h>
#include "UrduSorting.h"


UrduSorting::UrduSorting()
{
	MaxUrduCode=0x06FA; //Extended Arabic digit nine code +1
	MinUrduCode=0x0600;  // Arabic number sign
	CollationElement = NULL;
	currentPos=0;
	size=0;
	collationTableSize=0;
	int Size=MaxUrduCode-MinUrduCode;
	for(int i=0;i<Size;i++)
	{
		NormalizeArray[i]=false;
		CollationElementIndex[i]=-1;
	}
}

UrduSorting::~UrduSorting()
{
	Terminate();
}

bool UrduSorting::Initialize(string srfile)
{
	MaxUrduCode=0x06FA; //Extended Arabic digit nine code +1
	MinUrduCode=0x0600;  // Arabic number sign
	currentPos=0;
	size=0;
	collationTableSize=0;
	int Size=MaxUrduCode-MinUrduCode;
	for(int i=0;i<Size;i++)
	{
		NormalizeArray[i]=false;
		CollationElementIndex[i]=-1;
	}

	if (LoadCollationTable(srfile) == false)
		return false;
	
	return true;
}

void UrduSorting::Terminate()
{
	if (CollationElement != NULL)
	{
		delete [] CollationElement;
		CollationElement = NULL;
	}

}



bool UrduSorting::LoadCollationTable(string srfile)
{
	FileReader fr;
	if (fr.Open(srfile.c_str()) == false)
		return false;
	
	string buffer; 
	unsigned int ceCode=0;
	int size=0;
	
	if(fr.GetLine(buffer,3,'=')==true)
	{
		size=Hex2Dec(buffer);
		collationTableSize=size;
		CollationElement=new Code[size];	
	}
	else
		return false;
	try
	{
		
		for(int cIndex=0,index=0;fr.EoF()==false; )
		{
			if(fr.GetLine(buffer,512,'\n')==false)
				continue;

			if(fr.GetLine(buffer,512,']')==false)
				continue;//return false;

			ceCode=GetCharacterCode(buffer);
			
			if(fr.GetLine(buffer,512,']')==false)
				return false;
			int x=ceCode & 0xFFFF0000;
			if(x !=0 ) // To check Multiple char code e.g kaf + Dochashmi hay
			{
				index=ceCode>>16& 0x0000FFFF;
				//in the file Combining character should be come after its first letter
				//e.g  first kaf  has an entry and then kaf+do chashmi hay entry in //the file
				if(index>=MinUrduCode && index<=MaxUrduCode)
				{
					index=index-MinUrduCode;
					if(CollationElementIndex[index]==-1)
					{	
						CollationElementIndex[index]=cIndex;
						cIndex=cIndex+1;
					}
			
					CCode newEntry;
					newEntry.charCode=ceCode&0x0000FFFF;
					GetCollationCode(buffer,newEntry.collationCode,3);
					NormalizeArray[newEntry.charCode-MinUrduCode]=true;
					CollationElement[CollationElementIndex[index]].CElement.push_back(newEntry);
				}
			}
			else
			{
				index= ceCode & 0x0000FFFF;
				if (index>=MinUrduCode && index<=MaxUrduCode)
				{
					if(CollationElementIndex[index-MinUrduCode]=-1)
					{
						CollationElementIndex[index-MinUrduCode]=cIndex;
						cIndex=cIndex+1;

						CCode newEntry;
						newEntry.charCode=index;
						GetCollationCode(buffer,newEntry.collationCode,3);
						CollationElement[CollationElementIndex[index-MinUrduCode]].CElement.push_back(newEntry);					
					}
				}
				else
				{
					if(index==0x0000200C)  /// For non joiner 
					{	
						NonJoiner.charCode=index;
						GetCollationCode(buffer,NonJoiner.collationCode,3);
						//CollationElementTable[NonJoinerIndex].charCode=index;
						//CollationElementTable[NonJoinerIndex].collationCode=GetCollationCode(buffer);
					}
				}
			}
		}// End For
	}
	catch(IOException& ex)
	{
		fr.Close();
	}

	return true;
}


int UrduSorting::Hex2Dec(string hex)
{
	// string hex contains value as 20 having dec 32
	int pval=1;
	int val=0;
	for(int k=0;k<hex.length()-1;k++)
		pval*=16;
	
	for(int i=0;i<hex.length();i++)
	{
		if(hex[i]>='0' && hex[i]<='9')
			val+=pval*((int)hex[i]-48);
		else
			if(hex[i]>='A' && hex[i]<='F')
				val+=pval*((int)hex[i]-55);
		if(pval>=16)
			pval/=16;
	}
	return val;
}


unsigned int UrduSorting::GetCharacterCode(string hexstr)
{
	string hex="";
	int temp=hexstr.find(',',0);
	unsigned int value=0;
	if( temp > 0)
	{
		for(int i=1;i<temp;i++)
			hex+=hexstr[i];	
		value=Hex2Dec(hex);
		hex="";
		value=value<<16;
		value=value & 0xFFFF0000;
	}
	else
		temp=0;
	for(int i=temp+1;i<hexstr.length();i++)
		hex+=hexstr[i];
	
	value=value|Hex2Dec(hex);
	
	return value;
}

void UrduSorting::GetCollationCode(string hexstr,int CollationCode[],int levels)
{
	string hex="";
	
	for(int index=0,i=1;index<levels;index++)
	{
		for(;i<hexstr.length();i++)
		{
			if(hexstr[i]==',')
			{
				i++;
				break;
			}
			hex=hex+hexstr[i];
		}

		CollationCode[index]=Hex2Dec(hex);
		hex="";
	}	
}



string UrduSorting::GetCollationKey(wstring str, int strength)
{

	string level[3]={""};
	string CEKey="";
	bool endOfWord = false;
	int i;
	CCode code;
	
		Normalize(str);
		

		while (!endOfWord)
		{
			endOfWord=GetCharCollationCode(code.collationCode,strength);
		
			for(i=0;i<strength; i++)
				if(code.collationCode[i]!=0x00)
					level[i]+=code.collationCode[i];
			
		}	
	
		for(i=0; i<strength;i++)
			CEKey+=level[i];
		
		//CEKey+='\0';
		
	/*	cout<<"\n";
		for(i=0; i<CEKey.size(); i++)
			cout<<(int)CEKey[i]<<" ";
		cout<<"\n";*/

	return CEKey;

}


int UrduSorting::Compare(wstring source, wstring target, int strength)
{

	int result = GetCollationKey(source,strength).compare(GetCollationKey(target,strength));

	if(result == 0 && source.length()<target.length())
		result=-1;

	return result;
		
}

void UrduSorting::Normalize(wstring buff)
{
	wchar_t ChotiYeaHamza=0x0626;
	wchar_t BariYeaHamza=0x06D3;
	wchar_t WaoHamza=0x0624;
	wchar_t AlifHamzaAbove=0x0623;
	wchar_t AlifHamzaBelow=0x0625;
	cword = String2Wstring("");

	size=buff.length();
	currentPos=0;

	for(int i=0,j=0;i<size;i++,j++)
	{

		//cout<<buff[i]<<flush;
		if(buff[i]==ChotiYeaHamza)
		{
			if(i+1<size)
				cword+=0x0621; //character hamza
			else{
				cword+=0x06CC; //Choti yea 
				cword+=0x0654; //Diacritic hamza
			}
		}
		else if(buff[i]==BariYeaHamza)
		{
			cword+=0x06D2; //Bari yea 
			cword+=0x0654; //Diacritic hamza
		}
		else if(buff[i]==WaoHamza)
		{
			cword+=0x0648; //Wao 
			cword+=0x0654; //Diacritic hamza
		}
		else if(buff[i]==AlifHamzaAbove)
		{
			cword+=0x0627;
			cword+=0x0654;
		}
		else if(buff[i]==AlifHamzaBelow)
		{
			cword+=0x0627;
			cword+=0x0655;
		}
		else
			cword+=buff[i];
	}
	
}

bool UrduSorting::GetCharCollationCode(int CollationCode[],int levels)
{
	int nextpos=currentPos+1;

	if(currentPos>=size)
	{
		CollationCode[0]=0x00;//code="000"  // invalid character it must be ignore 
		CollationCode[1]=0x00;
		CollationCode[2]=0x00;
		return true;
	}
	else
	{	
		if(nextpos<size && cword[nextpos]>=MinUrduCode && cword[nextpos]<=MaxUrduCode 
			&& NormalizeArray[cword[nextpos]-MinUrduCode]==1
			&& cword[currentPos] >=MinUrduCode && cword[currentPos]<=MaxUrduCode
			&& CollationElementIndex[cword[currentPos]-MinUrduCode]!=-1)
		{
			int i;
			for (i=0;i<CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement.size() && CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement[i].charCode!= cword[nextpos] ; i++);

			if(i==CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement.size()) // Not a normalize case 
			{	
				for(int index=0;index<levels;index++)
					CollationCode[index]=CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement[0].collationCode[index];
				currentPos=currentPos+1; //consume 1 character
			}
			else
			{
				for(int index=0;index<levels;index++)
					CollationCode[index]=CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement[i].collationCode[index];
				currentPos=currentPos+2; //consume 2 characters				
			}
		}
		else
		{
			if(cword[currentPos] >=MinUrduCode && cword[currentPos]<=MaxUrduCode
				&& CollationElementIndex[cword[currentPos]-MinUrduCode]!=-1)
			{
				for(int index=0;index<levels;index++)
					CollationCode[index]=CollationElement[CollationElementIndex[cword[currentPos]-MinUrduCode]].CElement[0].collationCode[index];
				currentPos=currentPos+1; //consume 1 character
			}
			else
			{
				if(cword[currentPos]==NonJoiner.charCode)
				{
					for(int index=0;index<levels;index++)
						CollationCode[index]=NonJoiner.collationCode[index];
					currentPos=currentPos+1; //consume 1 character
				}
				else
				{
					if(cword[currentPos]==0x0020) // Space
					{
						currentPos=currentPos+1; //consume 1 character
						CollationCode[0]=0x41;
						CollationCode[1]=0x00;
						CollationCode[2]=0x00;
					}
					else						// Any other NonUrdu character
					{
						currentPos=currentPos+1; //consume 1 character
						CollationCode[0]=0x00;//code="000"  // invalid character it must be ignore 
						CollationCode[1]=0x00;
						CollationCode[2]=0x00;
					}
				}
			}
		}
	}
	return false;
}

