
#include "Utility.h"

// converts character array-based string to string object
// atMostNumChars tell maximum num of chars to be copied if null is not encounter
string CharString2String (const char* cString, int atMostNumChars)
{
	string resStr;
	for (int i = 0; i < atMostNumChars && cString[i] != 0; i++)
		resStr += cString[i];
	return resStr;
}

// converts ascii-based string (string) to unicode-based string (wstring)
wstring String2Wstring (string sString)
{
	wstring wsString;

	int n = sString.length();
	for (int i = 0; i < n; i++)
		wsString += (wchar_t)sString[i];

	return wsString;
}

// converts ascii-based string (string) to unicode-based string (wstring)
string WString2String (wstring wsString)
{
	string sString;

	int n = wsString.length();
	for (int i = 0; i < n; i++)
		sString += (char)wsString[i];

	return sString;
}

// converts unicode character string (wchar_t*) to unicode-based string (wstring)
wstring CharString2Wstring (const wchar_t* cString, int atMostNumChars)
{
	wstring wsString;
	for (int i = 0; i < atMostNumChars && cString[i] != 0; i++)
		wsString += (wchar_t)cString[i];

	return wsString;
}

// converts character string (char*) to unicode-based string (wstring)
wstring CharString2Wstring (const char* cString)
{
	wstring wsString;
	for (int i = 0; cString[i] != 0; i++)
		wsString += cString[i];

	return wsString;
}

wstring int2wstring (int num)
{
	char lineStr[20];
	sprintf(lineStr, "%i", num);
	wstring outStr = CharString2Wstring(lineStr);
	return outStr;
}

wstring int2UrduNumerals (int num)
{
	wstring outStr;
	char lineStr[20];
	sprintf(lineStr, "%i", num);

	int n = strlen(lineStr);
	int offset = 0x06F0-0x30;
	for (int i = 0; i < n; i++)
		outStr += offset + (wchar_t)lineStr[i];
	return outStr;
}

int wstring2int (wstring wstr)
{
	string str = WString2String(wstr);
	return atoi(str.data());
}

bool IsNumber (wstring str)
{
	for (int i = 0; i < str.size(); i++)
		if (str[i] < 0x0030 || str[i] > 0x0039)
			return false;

	return true;
}

// trims a given character string at start and end
char* Trim (char* str)
{
	int i = 0;
	while (IsWhitespace (str[i]))
		i++;

	int j = strlen(str)-1;
	while (IsWhitespace (str[j]))
		j--;

	int k = 0;
	int n = j-i+1;
	while(k < n)
		str[k++] = str[i++];

	str[k] = '\0';
	return str;
}

// trims a given string at start and end
string Trim (string& str)
{
	int i = 0;
	while (IsWhitespace (str[i]))
		i++;

	int j = str.length()-1;
	while (IsWhitespace(str[j]))
		j--;

	int n = j-i+1;
	str = str.substr(i, n);
	return str;
}

wstring Trim(wstring& wstr)
{
	int i = 0;
	while (IsWhitespace (wstr[i]))
		i++;

	int j = wstr.length()-1;
	while (IsWhitespace (wstr[j]))
		j--;

	int n = j-i+1;
	return (wstr = wstr.substr(i, n));
}

bool IsWhitespace (wchar_t ch)
{
	return (ch == 0x0020 || ch == 0x000A || ch == 0x000D  || ch == 0x0009);
}

bool DoesContainWildcardChars (wstring wstr)
{
	for (int i = 0; i < wstr.size(); i++)
		if (wstr[i] == 0x002A // '*' char
			|| wstr[i] == 0x003F) // '?' char
			return true;

	return false;
}

wstring RemoveWhitespaces (wstring str)
{
	wstring resStr;	
	for (int i = 0; i < str.length(); i++){
		if (!(str[i] == __SPACE || str[i] == __ZWNJ || str[i] == __ZWJ
				|| str[i] == 0x000D || str[i] == 0x000A || str[i] == 0x0009))
			resStr += str[i];
	}
	return resStr;
}

void ConvertLittleEndian2BigEndian (const wstring& uniStr, wstring& utfStr)
{
	wchar_t wc;
	for(int i = 0; i < uniStr.length(); i++)
	{
		if (__ULE_UCS_SIZE == __ULE_UCS_2)
			wc = (uniStr[i] >> 8) | (uniStr[i] << 8);
		else // if (__ULE_UCS_SIZE == __ULE_UCS_4)
		{
			const Byte	B1 = (unsigned char)0x000000FF,
						B2 = (unsigned char)0x0000FF00,
						B3 = (unsigned char)0x00FF0000,
						B4 = (unsigned char)0xFF000000;

			wc  = ((uniStr[i] & B4) >> 24);
			wc |= ((uniStr[i] & B3) >> 8);
			wc |= ((uniStr[i] & B2) << 8);
			wc |= ((uniStr[i] & B1) << 24);
		}
		utfStr+=wc;
	}

	return;
}
