
#ifndef __ULE_UTILITY_H__
#define __ULE_UTILITY_H__

#include <string>
#include <vector>
using namespace std;




enum {FILE_NOT_FOUND=0x10, FILE_READ_ERROR=0x11};
typedef unsigned long	DWord;
typedef unsigned short	Word;
typedef unsigned char	Byte;


typedef unsigned short			uleChar;	// wide char (2-byte) stores unicode char
typedef basic_string <uleChar>	uleString;	// wide string stores unicode string

// enums
enum __ULE_UNICODE_FILE_FORMAT {__ULE_UCS_2 = 2, __ULE_UCS_4 = 4};
const __ULE_UNICODE_FILE_FORMAT __ULE_UCS_SIZE = __ULE_UCS_2;

// consts
const Word	__ULE_UCS_2_BOM = 0xFEFF; // "Byte Order Market" for UCS-2
const DWord	__ULE_UCS_4_BOM = 0x0000FEFF; // "Byte Order Market" for UCS-4

const DWord	__ULE_UCS_BOM	= __ULE_UCS_2_BOM;

const Byte __ULE_WCHAR_SIZE = sizeof(wchar_t);
const bool __ULE_LONG_WCHAR = ( (__ULE_WCHAR_SIZE == __ULE_UCS_4) ? true : false );
const Byte	__ULE_UCS_BOM_SIZE	= ( (__ULE_UCS_BOM == __ULE_UCS_2_BOM) ? 2 : 4 );

const int __ULE_FILE_WCHAR_SIZE = 2; // byte 

const wchar_t 
	__SPACE			= 0x0020,
	__ASTERISK		= 0x002A,
	__QUEST_MARK	= 0x003F,
	__ZWNJ			= 0x200C,
	__ZWJ			= 0x200D;

// char string, STL string, wstring interconversion routines
string CharString2String (const char* cString, int atMostNumChars);
wstring CharString2Wstring (const wchar_t* cString, int atMostNumChars);
wstring String2Wstring (string);
string WString2String (wstring);
wstring CharString2Wstring (const char* cString);

// integers related string functions 
bool IsNumber (wstring str);
wstring int2wstring (int num);
wstring int2UrduNumerals (int num);
int wstring2int (wstring str);

// string manipulation routines
char* Trim (char* str);
string Trim (string& str);
wstring Trim (wstring& wstr);
wstring RemoveWhitespaces (wstring str);

inline wstring& ReplaceCharW (wstring& str, 
		const wchar_t cReplaceIt, const wchar_t cReplaceWith)
{
	int n = str.size();
	for (int i = 0; i < n; i++)
		if (str[i] == cReplaceIt)
			str[i] = cReplaceWith;
	return str;
}

// others
bool IsWhitespace (wchar_t ch);
bool DoesContainWildcardChars (wstring wstr);

void ConvertLittleEndian2BigEndian (const wstring& uniStr, wstring& utfStr);

#endif // __ULE_UTILITY_H__
